/*
 * RDATE.NLM - an implementation of the RDATE command found on
 * many unix systems; Inspired by Brad Clements' RDATE.NLM, except:
 *   - produces no ouput, except in debug mode
 *   - does not unload itself if we don't get a reply
 *   - Simpler interface
 */

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>	/* timeval */
#include <sys/socket.h>
#include <sys/filio.h>	/* FIONBIO */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <io.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include	<nwenvrn.h>

#define	DEFAULT_SLEEP	60*5
#define	TIME_PORT		37
#define	TIME_OUT			1
#define	MAXDIFF			2

void	RdateEnd(void);

char           *myname,*pmy;
int				 debug;
int				 Socket;
int		iTimeout = DEFAULT_SLEEP;

struct sockaddr_in server_sockaddr;

void Usage(void)
{
	ConsolePrintf("usage: %s -t <interval> <server_address>\r\n", myname);
	ConsolePrintf("\tdefault interval is %d seconds\r\n",DEFAULT_SLEEP);
	exit(1);
};

main(argc, argv)
	int             argc;
	char          **argv;
{
	int             rc,
	                arg,
	                i;
	struct timeval  tv;
	fd_set          rfds;
					 
	time_t			 rtime,
						 delta,
						 now;

	struct tm		*ptm;
	char				 tbuf[60];

	debug=0;			/* no debugging */
	Socket=0;		/* so Exit routine does not close a random socket */
	

/* no screen, use the system console */
	i=CreateScreen("System Console",0);
	SetCurrentScreen(i);
	DisplayScreen(i);

/* tidy up if we exit() or are unloaded */
	ConsolePrintf("RDATE.NLM version %s Copyright (c) 1993 University of Salford\r\n",VERSION);
	atexit(RdateEnd);

/* parse the command line */
	myname = *argv;
	if (pmy=strrchr(myname,'\\') ) myname=pmy+1;
		else	if (pmy=strrchr(myname,':'))  myname=pmy+1;

for (i=1;i<argc;i++)
	if (strcmp(argv[i],"-d")==0) debug++;
		else if (strcmp(argv[i],"-t")==0) {
			if (argv[i+1]==NULL) Usage();
			iTimeout=atoi(argv[i+1]);
			if (iTimeout==0) Usage();
			i++;
			continue;
			} 
			else if (*argv[i]=='-') Usage();
				else break;
	
	if (i == argc) Usage();


	server_sockaddr.sin_addr.s_addr = inet_addr(argv[i]);
	if (server_sockaddr.sin_addr.s_addr == -1) {
		ConsolePrintf("%s: %s is a malformed address\r\n", myname, argv[i]);
		exit(1);
	}
	server_sockaddr.sin_family = AF_INET;
	server_sockaddr.sin_port = htons(TIME_PORT);

/* create the UDP sockets */
	Socket = socket(PF_INET, SOCK_DGRAM, 0);
	if (Socket == -1) {
		perror("socket");
		exit(1);
	}

/* set socket to non-blocking */
	arg = 1;
	rc = ioctl(Socket, FIONBIO, (char *) &arg);
	if (rc == -1) {
		perror("ioctl/FIONBIO");
		exit(1);
	}

	for (;;) {
/*  send a packet the same size as we want back */
		rc = sendto(Socket, (char *) &rtime, sizeof(rtime) , 0,
			    (struct sockaddr *) & server_sockaddr,
			    sizeof (server_sockaddr));
		if (rc == -1) {
			perror("sendto");
			exit(1);
		}

		FD_ZERO(&rfds);
		FD_SET(Socket, &rfds);
		tv.tv_sec = 1;
		tv.tv_usec = 0;
		rc = select(FD_SETSIZE, &rfds, (fd_set *) NULL, (fd_set *) NULL,
			    &tv);
		if (rc == -1) {
			perror("select");
			exit(1);
		}
		if (rc == 0)
			continue;

		rc = recv(Socket,(char *) &rtime, sizeof (rtime), 0);
		if (rc == -1) {
			perror("recv");
			exit(1);
		}

/* correct to time since 1970, rather than 1900 */
		rtime=ntohl(rtime) - (u_long)2208988800l;
		now=time(NULL);
		delta=abs(now-rtime);
		if (delta>MAXDIFF) {
				if (debug) {
					ptm=localtime(&now);
					strftime(tbuf,sizeof(tbuf),"%A %d %b, %Y %T %Z",ptm);
					ConsolePrintf("\r\nTime was %s\r\n",tbuf);
					};

				ptm=localtime(&rtime);
				strftime(tbuf,sizeof(tbuf),"%A %d %b, %Y  %T %Z",ptm);
				ConsolePrintf("Time set to %s (%d seconds change)\r\n",tbuf,delta);

				SetFileServerDateAndTime(
						ptm->tm_year, ptm->tm_mon+1, ptm->tm_mday,
						ptm->tm_hour ,ptm->tm_min, ptm->tm_sec);

				if (debug) {
					now=time(NULL);
					ptm=localtime(&now);
					strftime(tbuf,sizeof(tbuf),"%A %d %b, %Y %T %Z",ptm);
					ConsolePrintf("Time is now %s\r\n",tbuf);
					};
				};	

/* SLEEP for iTimeout Seconds */
		tv.tv_sec = iTimeout;
		tv.tv_usec = 0;
		rc = select(FD_SETSIZE, (fd_set *) NULL, (fd_set *) NULL, 
						(fd_set *) NULL, &tv);
		if (rc == -1) {
			perror("select");
			exit(1);
		}

/* collect garbage on the net... */
		for (;;) {
		rc = recv(Socket,(char *) &rtime, sizeof (rtime), 0);
		if ((rc==-1) & (errno==EWOULDBLOCK)) break;
	
		if (rc == -1 ) {
			perror("recv");
			exit(1);
			};

		ConsolePrintf("%s spurious time response\r\n",myname);
		};
		if (debug) ConsolePrintf("+");
	}
}

void	RdateEnd(void)
{
int rc;

if (Socket) {
		rc = close(Socket);
		if (rc == -1) {
			perror("close");
			exit(1);
		};
	};
ConsolePrintf("%s: NLM Exiting\r\n", myname);
}